package net.pricefx.common.api;

import java.util.HashMap;
import java.util.Map;

/**
 * Tokens for page to page navigation. <br/>
 * Do not use "/" in page tokens because gwtplatform handles them as separators
 * of tokens and some functionality might be then broken; for instance back
 * button navigation.
 *
 * NOTE: for the Groovy API a new net.pricefx.formulaengine.scripting.AppPages interface holds all those
 * page tokens that are confirmed to work in Unity UI. I doubt we'll added many more tokens here... but
 * if we do, we should also add them in AppPages...
 *
 * @author Dusan Skopik
 *
 */
public final class PageTokens {

	public static final String MAIN_PAGE = "mainPage";
	public static final String START_PAGE = "startPage";
	public static final String PRODUCTS_PAGE = "productsPage";
	public static final String PRODUCTEXTENSIONS_PAGE = "productExtensionsPage";
	public static final String COMPETITION_PAGE = "competitionPage";
	public static final String BOM_PAGE = "bomPage";
	public static final String PRODUCTREFERENCES_PAGE = "xrefsPage";
	public static final String PRICE_CALCULATION_PAGE = "priceCalculationPage";
	public static final String PRICE_RECORDS_PAGE = "priceRecordsPage";
	public static final String CLAIM_RECORDS_PAGE = "claimRecordsPage";
	public static final String CUSTOMERS_PAGE = "customersPage";
	public static final String CUSTOMEREXTENSIONS_PAGE = "customerExtensionsPage";
	public static final String PRICING_PARAMETERS_PAGE = "pricingParametersPage";
	public static final String PRICE_LIST_PAGE = "priceListPage";
	public static final String PRICE_LIST_TYPE_PAGE = "priceListTypePage";
	public static final String MANUAL_PRICE_LIST_PAGE = "manualPriceListPage";
	public static final String PRICE_GRID_PAGE = "priceGridPage";
	public static final String LOGIN_PAGE = "loginPage";
	public static final String PRICE_SHOP_PAGE = "priceShopPage";
	public static final String QUOTES_MASS_UPDATE_PAGE = "quotesMassUpdatePage";
	public static final String QUOTETYPES_PAGE = "quoteTypesPage";
    public static final String QUOTES_NEW_PAGE = "newQuotePage";
	public static final String CONTRACTS_MASS_UPDATE_PAGE = "contractsMassUpdatePage";
	public static final String USER_PAGE = "userPage";
	public static final String USERGROUPS_PAGE = "userGroupsPage";
	public static final String BUSINESSROLES_PAGE = "businessRolesPage";
	public static final String TFATOKENS_PAGE = "tfaTokensPage";
	public static final String SYSADMIN_SETTINGS_PAGE = "sysadminSettingsPage";
	public static final String PARTITON_ADMIN_PAGE = "partitionAdminPage";
	public static final String APIKEY_ADMIN_PAGE = "apikeyAdminPage";
	public static final String CLUSTER_ADMIN_PAGE = "clusterAdminPage";
	public static final String DBMIGRATION_ADMIN_PAGE = "dbmigrationAdminPage";
	public static final String SERVERMIGRATION_ADMIN_PAGE = "servermigrationAdminPage";
	public static final String HOSTINFO_PAGE = "hostInfoPage";
	public static final String RUNCOMMAND_PAGE = "runCommandPage";
	public static final String VIEWLOG_PAGE = "viewLogPage";
	public static final String CONFIG_WIZARDS_PAGE = "configWizardsPage";
	public static final String CONFIG_WIZARD_PAGE = "configWizardPage";

	public static final String DATA_MANAGER_PAGE = "dataManagerPage";
	public static final String DATAFEEDS_PAGE = "dataFeedsPage";
	public static final String DATASOURCES_PAGE = "dataSourcesPage";
	public static final String DATAMARTS_PAGE = "datamartsPage";
	public static final String DATALOAD_CALCULATIONLOGIC_PAGE = "dataLoadCalculationLogicPage";
	public static final String DATALOADS_PAGE = "dataLoadsPage";
	public static final String DIST_PACALC_LOGIC_PAGE = "distPACalcLogicPage";

	public static final String DATAMART_PAGE = "datamartPage";
	public static final String QUERY_CALCULATIONLOGIC_PAGE = "queryCalculationLogicPage";
	public static final String PA_SIMULATIONS_PAGE = "paSimulationsPage";
	public static final String PA_ROLLUPS_PAGE = "paRollupsPage";
	public static final String DATA_ANALYZER_PAGE = "dataAnalyzerPage";
	public static final String SIMULATION_PAGE = "simulationPage";

	public static final String CONFIGURATION_PAGE = "configurationPage";
	public static final String VIEWPARTITIONAUDITLOG_PAGE = "viewPartitionLogPage";
	public static final String WORKFLOW_BUILDER_PAGE = "workflowBuilderPage";
	public static final String WORKFLOW_DELEGATION_PAGE = "workflowDelegationPage";
	public static final String WORKFLOWS_PAGE = "workflowPage";
	public static final String CALCULATEDFIELDSETS_PAGE = "calculatedFieldsPage";
	public static final String DASHBOARDADMIN_PAGE = "dashboardAdminPage";
	public static final String I18NADMIN_PAGE = "i18nAdminPage";
	public static final String DASHBOARD_PAGE = "dashboardPage";
	public static final String DASHBOARD_EMBEDDED_TABBED = "dashboardEmbeddedTabbed";
	public static final String DASHBOARD_EMBEDDED_WINDOWED = "dashboardEmbeddedWindowed";
	public static final String REBATESTYPES_PAGE = "rebateTypesPage";
	public static final String REBATEAGREEMENTTEMPLATES_PAGE = "rebateAgreementTemplatesPage";
	public static final String REBATEAGREEMENTS_PAGE = "rebateAgreementsPage";
	public static final String REBATE_CALCULATIONLOGIC_PAGE = "rebateCalculationLogicPage";
	public static final String REBATE_CALCULATIONS_PAGE = "rebateCalculationsPage";
	public static final String REBATE_RECORDS_PAGE = "rebateRecordsPage";
	public static final String PAYOUT_RECORDS_PAGE = "payoutRecordsPage";
	public static final String CONTRACTTERMTYPES_PAGE = "contractTermTypesPage";
	public static final String CLAIMTYPES_PAGE = "claimTypesPage";
	public static final String CONTRACTS_PAGE = "contractsPage";
	public static final String CLAIMS_PAGE = "claimsPage";
	public static final String CONTRACT_CALCULATION_PAGE = "contractCalculationPage";
	public static final String DCR_PAGE = "dataChangeRequestPage";
	public static final String TODO_ADMIN_PAGE = "todoAdminPage";
	public static final String GROOVYLIB_ADMIN_PAGE = "groovyLibAdminPage";
	public static final String EVENT_ADMIN_PAGE = "eventAdminPage";
	public static final String EMAIL_ADMIN_PAGE = "emailAdminPage";
	public static final String PD_FORMULAS_CONFIGURATION_PAGE = "pdFormulasConfPage";
	public static final String CD_FORMULAS_CONFIGURATION_PAGE = "cdFormulasConfPage";
	public static final String QH_FORMULAS_CONFIGURATION_PAGE = "qhFormulasConfPage";
	public static final String QPF_FORMULAS_CONFIGURATION_PAGE = "qpfFormulasConfPage";
	public static final String CTTF_FORMULAS_CONFIGURATION_PAGE = "cttfFormulasConfPage";
	public static final String RBTF_FORMULAS_CONFIGURATION_PAGE = "rbtfFormulasConfPage";
	public static final String CH_FORMULAS_CONFIGURATION_PAGE = "chFormulasConfPage";
	public static final String RAH_FORMULAS_CONFIGURATION_PAGE = "rahFormulasConfPage";
	public static final String RAT_FORMULAS_CONFIGURATION_PAGE = "ratFormulasConfPage";
	public static final String CF_FORMULAS_CONFIGURATION_PAGE = "cfFormulasConfPage";
	public static final String PL_LPG_FORMULAS_CONFIGURATION_PAGE = "plLpgFormulasConfPage";
	public static final String JOBTRACKER_PAGE = "jobTrackerPage";
	public static final String DATAIMPORTEXPORT_PAGE = "dataImportExportPage";


	public static final String CHART_FORMATING = "chartFormating";
	public static final String GRID_PREFERENCES = "gridPreferences";

	public static final String PA_EXCELCLIENT = "paExcelClient";

	// Optimization
	public static final String MODEL_FORMULAS_PAGE = "modelFormulasPage";
	public static final String MODELS_PAGE = "modelsPage";
	public static final String SIMPLEMODELS_PAGE = "simpleModelsPage";
	public static final String MODELRECORDS_PAGE = "modelRecordsPage";
	public static final String MODELTYPES_PAGE = "modelTypesPage";

	// sub tokens used for identification of analytics
	public static final String DATA_ANALYZER_PAGE_SCATTER = "dataAnalyzerPageScatter";
	public static final String DATA_ANALYZER_PAGE_TIMESERIES_SCATTER = "dataAnalyzerPageTimeSeriesScatter";
	public static final String DATA_ANALYZER_PAGE_WATERFALL = "dataAnalyzerPageWaterfall";
	public static final String DATA_ANALYZER_PAGE_WATERFALLCOMPARISON = "dataAnalyzerPageWaterfallComparison";
	public static final String DATA_ANALYZER_PAGE_BUBBLE = "dataAnalyzerPageBubble";
	public static final String DATA_ANALYZER_PAGE_PIE = "dataAnalyzerPagePie";
	public static final String DATA_ANALYZER_PAGE_BARCHART = "dataAnalyzerPageBarChart";
	public static final String DATA_ANALYZER_PAGE_TIMESERIES = "dataAnalyzerPageTimeSeries";
	public static final String DATA_ANALYZER_PAGE_TIMESERIES_DATETIME = "dataAnalyzerPageTimeSeriesDatetime";
	public static final String DATA_ANALYZER_PAGE_HISTOGRAM = "dataAnalyzerPageHistogram";
	public static final String DATA_ANALYZER_PAGE_BOXPLOT = "dataAnalyzerPageBoxplot";
	public static final String DATA_ANALYZER_PAGE_HEATMAP = "dataAnalyzerPageHeatmap";
	public static final String DATA_ANALYZER_PAGE_DATATABLE = "dataAnalyzerPageDataTable";

	private static Map<String,String> jumpPages ;

	/**
	 * Gets a map of <pageId, i18n-token> for pages that are listed in a page selector (e.g. for ToDo)
	 * We cannot just use all pages as some do not make sense and some shouldn't be included there
	 *
	 * @return
	 */
	public static Map<String,String> getJumpPages() {
		if(jumpPages != null) return jumpPages;

		jumpPages = new HashMap<>();
		jumpPages.put(COMPETITION_PAGE, "main_competitionTitle");
		jumpPages.put(BOM_PAGE, "main_bomTitle");
		jumpPages.put(PRODUCTS_PAGE, "main_productsTitle");
		jumpPages.put(PRODUCTEXTENSIONS_PAGE, "main_productExtensionsTitle");
		jumpPages.put(CUSTOMERS_PAGE, "main_customersTitle");
		jumpPages.put(CUSTOMEREXTENSIONS_PAGE, "main_customerExtensionsTitle");
		jumpPages.put(PRICING_PARAMETERS_PAGE, "main_pricingParametersTitle");
		jumpPages.put(PRICE_LIST_PAGE, "main_priceListTitle");
		jumpPages.put(MANUAL_PRICE_LIST_PAGE, "main_manualPriceListTitle");
		jumpPages.put(PRICE_GRID_PAGE, "main_priceGridTitle");
		jumpPages.put(PRICE_SHOP_PAGE, "main_quotes");
		jumpPages.put(SIMULATION_PAGE, "main_pb_simulationTitle");
		jumpPages.put(REBATEAGREEMENTS_PAGE, "main_rebateAgreementsTitle");
		jumpPages.put(DATA_ANALYZER_PAGE_SCATTER, "main_dataAnalyzerScatter");
		jumpPages.put(DATA_ANALYZER_PAGE_TIMESERIES_SCATTER, "main_dataAnalyzerTimeSeriesScatter");
		jumpPages.put(DATA_ANALYZER_PAGE_WATERFALL, "main_dataAnalyzerWaterfall");
		jumpPages.put(DATA_ANALYZER_PAGE_WATERFALLCOMPARISON, "main_dataAnalyzerWaterfallComparison");
		jumpPages.put(DATA_ANALYZER_PAGE_BUBBLE, "main_dataAnalyzerBubble");
		jumpPages.put(DATA_ANALYZER_PAGE_PIE, "main_dataAnalyzerPie");
		jumpPages.put(DATA_ANALYZER_PAGE_BARCHART, "main_dataAnalyzerBarAndLine");
		jumpPages.put(DATA_ANALYZER_PAGE_TIMESERIES, "main_dataAnalyzerTimeSeries");
		jumpPages.put(DATA_ANALYZER_PAGE_TIMESERIES_DATETIME, "main_dataAnalyzerDatetimeTimeSeries");
		jumpPages.put(DATA_ANALYZER_PAGE_HISTOGRAM, "main_dataAnalyzerHistogram");
		jumpPages.put(DATA_ANALYZER_PAGE_BOXPLOT, "main_dataAnalyzerBoxplot");


		return jumpPages;

	}

}
