package net.pricefx.domain;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.pricefx.common.api.PriceFxInterface;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.googlecode.genericdao.search.Filter;
import net.pricefx.logic.core.Utils;

/**
 * SellerGroup
 */
public class SellerGroup {

    private String label;
    private String sellerFieldName;
    private String sellerFieldLabel;
    private String sellerFieldValue;
    private String sellerFilterCriteria;
    private Map<String, Object> sellerFilterCriteriaMap;

    // TODO
//    /**
//     * Returns the seller field label when an item (not a filter statement) from the seller group picker is selected. The method is connected to the input field created by {@link net.pricefx.formulaengine.scripting.inputbuilder.InputBuilderFactory#createSellerGroupEntry(String) createSellerGroupEntry}.<p>
//     * Example:
//     * <blockquote>
//     * <pre>
//     * def inputMap = api.input("SellerGroup")
//     * if (inputMap == null){
//     *   return
//     * }
//     *
//     * def input = SellerGroup.fromMap(inputMap)
//     * String inputLabel = input.getLabel()
//     *
//     * return inputLabel
//     * </pre>
//     * </blockquote>
//     *
//     * @return The label value.
//     */
    public String getLabel() {
        return label;
    }

    /**
     * Sets the new Seller Group input label.<br>
     * Example:
     * <blockquote>
     * <pre>
     * SellerGroup cg = api.sellerGroupEntry("SellerGroup")
     * cg.setLabel("The new seller group label")
     * </pre>
     * </blockquote>
     *
     * @param label The new label of the Seller Group.
     */
    public void setLabel(String label) {
        // fix: PFCD-3022 - silently truncate the label to fit into database
        if (label != null && label.length() > 255) {
            label = label.substring(0, 254) + "\u2026" /* HORIZONTAL ELLIPSIS */;
        }
        this.label = label;
    }

    // TODO
//    /**
//     * Returns the seller field name when an item (not a filter statement) from the seller group picker is selected. The method is connected to the input field created by {@link net.pricefx.formulaengine.scripting.inputbuilder.InputBuilderFactory#createSellerGroupEntry(String) createSellerGroupEntry}.<p>
//     * Example:
//     * <blockquote>
//     * <pre>
//     * def inputMap = api.input("SellerGroup")
//     * if (inputMap == null){
//     *   return
//     * }
//     *
//     * def input = SellerGroup.fromMap(inputMap)
//     * String inputName = input.getSellerFieldName()
//     *
//     * return inputName
//     * </pre>
//     * </blockquote>
//     *
//     * @return The sellerFieldName value.
//     */
    public String getSellerFieldName() {
        return sellerFieldName;
    }

    /**
     * Sets a field name value of the SellerGroup object.<br>
     * Example:
     * <blockquote>
     * <pre>
     * SellerGroup sellers = new SellerGroup()
     * sellers.setSellerFieldName("sellerId")
     * </pre>
     * </blockquote>
     *
     * @param sellerFieldName The value of the seller field name.
     */
    public void setSellerFieldName(String sellerFieldName) {
        this.sellerFieldName = sellerFieldName;
    }

    // TODO
//    /**
//     * Returns the seller field label when an item (not a filter statement) from the seller group picker is selected. The method is connected to the input field created by {@link net.pricefx.formulaengine.scripting.inputbuilder.InputBuilderFactory#createSellerGroupEntry(String) createSellerGroupEntry}.<p>
//     * Example:
//     * <blockquote>
//     * <pre>
//     * def inputMap = api.input("SellerGroup")
//     * if (inputMap == null){
//     *   return
//     * }
//     *
//     * def input = SellerGroup.fromMap(inputMap)
//     * String inputLabel = input.getSellerFieldLabel()
//     *
//     * return inputLabel
//     * </pre>
//     * </blockquote>
//     *
//     * @return The value of the label for the SellerGroup object.
//     */
    public String getSellerFieldLabel() {
        return sellerFieldLabel;
    }

    /**
     * Sets a field label value of the SellerGroup object.<br>
     * Example:
     * <blockquote>
     * <pre>
     *  SellerGroup sellers = new SellerGroup()
     *  sellers.setSellerFieldLabel("The Seller Label")
     * </pre>
     * </blockquote>
     *
     * @param sellerFieldLabel The value of the seller field label for the SellerGroup object.
     */
    public void setSellerFieldLabel(String sellerFieldLabel) {
        this.sellerFieldLabel = sellerFieldLabel;
    }

    // TODO
//    /**
//     * Returns the seller field value when an item (not a filter statement) from the seller group picker is selected. The method is connected to the input field created by {@link net.pricefx.formulaengine.scripting.inputbuilder.InputBuilderFactory#createSellerGroupEntry(String) createSellerGroupEntry}.<p>
//     * Example:
//     * <blockquote>
//     * <pre>
//     * def inputMap = api.input("SellerGroup")
//     * if (inputMap == null){
//     *   return
//     * }
//     *
//     * def input = SellerGroup.fromMap(inputMap)
//     * String inputValue = input.getSellerFieldValue()
//     *
//     * return inputValue
//     * </pre>
//     * </blockquote>
//     *
//     * @return The sellerFieldValue value.
//     */
    public String getSellerFieldValue() {
        return sellerFieldValue;
    }

    /**
     * Sets a field value of the SellerGroup object.<br>
     * Example:
     * <blockquote>
     * <pre>
     * SellerGroup sellers = new SellerGroup()
     * sellers.setSellerFieldValue("123456789")
     * </pre>
     * </blockquote>
     *
     * @param sellerFieldValue The value of the field for the SellerGroup object.
     */
    public void setSellerFieldValue(String sellerFieldValue) {
        this.sellerFieldValue = sellerFieldValue;
    }

    // TODO
//    /**
//     * Returns a filter as deserialized JSON by a criteria selected from the seller group picker. The method is connected to the input field created by {@link net.pricefx.formulaengine.scripting.inputbuilder.InputBuilderFactory#createSellerGroupEntry(String) createSellerGroupEntry}.<p>
//     * Example:
//     * <blockquote>
//     * <pre>
//     * def inputMap = api.input("SellerGroup")
//     * if (inputMap == null){
//     *   return
//     * }
//     *
//     * def input = SellerGroup.fromMap(inputMap)
//     * String inputFilterJson = input.getSellerFilterCriteria()
//     *
//     * return inputFilterJson
//     * </pre>
//     * </blockquote>
//     *
//     * @return The deserialized JSON formatted filter object.
//     */
    @JsonIgnore
    protected String getSellerFilterCriteriaJson() {
        return sellerFilterCriteria;
    }

    /**
     * Sets the filter criteria from a JSON filter for the SellerGroup.<br>
     * Example:
     * <blockquote>
     * <pre>
     * def inputMap = api.input("SellerGroup")
     * 	  if (inputMap == null){
     * 	   return
     *            }
     * def input = SellerGroup.fromMap(inputMap)
     * Map filterJson = """{
     *    "_constructor":"AdvancedCriteria",
     *    "operator":"and",
     *    "criteria":[
     *       {
     *          "fieldName":"sellerId",
     *          "operator":"iEquals",
     *          "value":"123456789"
     *       }
     *    ]
     * }
     * """
     *
     * SellerGroup newsg = new SellerGroup()
     * newsg.setSellerFilterCriteriaJson(filterJson)
     * return newsg
     *     </pre>
     * </blockquote>
     *
     * @param sellerFilterCriteria The filter criteria as a JSON representation.
     */
    @JsonIgnore
    protected void setSellerFilterCriteriaJson(String sellerFilterCriteria) {
        this.sellerFilterCriteria = sellerFilterCriteria;
    }

    // TODO
//    /**
//     * Returns a filter as JSON by a criteria selected from the seller group picker. The method is connected to the input field created by {@link net.pricefx.formulaengine.scripting.inputbuilder.InputBuilderFactory#createSellerGroupEntry(String) createSellerGroupEntry}.<p>
//     * Example:
//     * <blockquote>
//     * <pre>
//     * def inputMap = api.input("SellerGroup")
//     * if (inputMap == null){
//     *   return
//     * }
//     *
//     * def input = SellerGroup.fromMap(inputMap)
//     * String inputFilterJson = input.getSellerFilterCriteriaJson()
//     *
//     * return inputFilterJson
//     * </pre>
//     * </blockquote>
//     *
//     * @return The JSON formatted filter.
//     */
    public Map<String, Object> getSellerFilterCriteria() {
        if (sellerFilterCriteriaMap == null && sellerFilterCriteria != null) {
//            sellerFilterCriteriaMap = jsonDeserialize(sellerFilterCriteria, new HashMap<>());
        }
        return sellerFilterCriteriaMap;
    }

    /**
     * Sets the filter criteria from a map of filters for the SellerGroup.<br>
     * Example:
     * <blockquote>
     * <pre>
     * def inputMap = api.input("SellerGroup")
     * 	  if (inputMap == null){
     * 	   return
     *            }
     * def input = SellerGroup.fromMap(inputMap)
     * Map filterJson = [_constructor: "AdvancedCriteria",
     *                                  operator    : "and",
     *                                  criteria    : [
     *                                          [
     *                                                  fieldName   : "sellerId",
     *                                                  operator    : "equals",
     *                                                  value       : "123456789",
     *                                                  _constructor: "AdvancedCriteria"
     *                                          ]
     *                                  ]
     *     ]
     *
     *
     * SellerGroup newsg = new SellerGroup()
     * newsg.setSellerFilterCriteria(filterJson)
     * return newsg
     *     </pre>
     * </blockquote>
     *
     * @param sellerFilterCriteriaMap The seller filter criteria as a map.
     */
    public void setSellerFilterCriteria(Map<String, Object> sellerFilterCriteriaMap) {
        this.sellerFilterCriteriaMap = sellerFilterCriteriaMap;
//        this.sellerFilterCriteria = (sellerFilterCriteriaMap == null) ? null : jsonSerialize(sellerFilterCriteriaMap);
    }

    /**
     * Converts SellerGroup to a filter object. Creates Filter from instance properties sellerFieldName and sellerFieldValue, or from sellerCriteria. Can return null.<p>
     * Example:
     * <blockquote>
     * <pre>
     * def sellerGroupMap = api.input("SellerGroup")
     * if (sellerGroupMap == null){
     *   return
     * }
     *
     * def sellers = api.find("P", (input.SellerGroup as SellerGroup)?.asFilter())
     * return sellers
     * </pre>
     * </blockquote>
     *
     * @return The filter object.
     */
    @JsonIgnore
    public Filter asFilter() {
        if (sellerFieldName != null && sellerFieldValue != null) {
            return new Filter(sellerFieldName, sellerFieldValue);
        } else if (sellerFilterCriteria != null) {
//            return DMFilter.buildAdvancedCriteriaFilter(getSellerFilterCriteria());
        }
        return null;
    }

    /**
     * Converts SellerGroup into a filter including converting any SX searchable items into a custom filter. Not intended for PA.<p>
     * Example:
     * <blockquote>
     * <pre>
     * def sellerGroup = api.input("SellerGroup")
     *
     * SellerGroup sg = SellerGroup.fromMap(api.input("Entry"))
     * def sellers = api.find("SL", sg.asFilterWithSXSubQuery())
     * </pre>
     * </blockquote>
     *
     * @return The filter object.
     */
    @JsonIgnore
    public Filter asFilterWithSXSubQuery() {
        List<Filter> filters = new ArrayList<>(Collections.singleton(asFilter()));

        if (filters.get(0) != null) {
//            ExtensionUtils.replaceFiltersBySubQuery(PriceFxInterface.FIELD_SELLER_ID, AbstractSellerExtension.class, filters);
        }
        return filters.get(0);
    }

    /**
     * Returns resolved filter using the given Data Source, which should represent the Seller attributed object class.
     *
     * @param sellerDS Seller Data Source.
     * @return The filter object.
     * @throws DMFieldNotFoundException
     */
    public Filter asFilter(final DMDataSource sellerDS) throws Exception {
        return asFilter(sellerDS, Collections.emptySet());
    }

    /**
     * Returns resolved filter using the given Seller Data Source and external Data Source, which should represent the Seller attributed object class.
     *
     * @param sellerDS Seller Data Source. Must not be null.
     * @param pxExtDS   External Data Source.
     * @return The filter object.
     * @throws DMFieldNotFoundException
     */
    public Filter asFilter(final DMDataSource sellerDS, final Collection<DMDataSource> pxExtDS) throws Exception {
        if (sellerDS == null) {
            throw new IllegalArgumentException("SellerDS argument must not be null");
        }

        return null;
    }

    /**
     * Creates SellerGroup object from a Map object that represents SellerGroup.<br>
     * Allows you, for example, to use this object in a Datamart query with usage of the "where" clause.<p>
     * Example:
     * <blockquote>
     * <pre>
     * def sgInput = api.sellerGroupEntry("Entry")
     * SellerGroup cg = SellerGroup.fromMap(sgInput)
     *     </pre>
     * </blockquote>
     *
     * @param map The representation of SellerGroup as a Map.
     * @return The SellerGroup object.
     */
    public static SellerGroup fromMap(final Map<String, Object> map) {
        return fromMap(Utils.getObjectMapper(), map);
    }

    /**
     * Creates SellerGroup object from a Map object that represents SellerGroup.
     *
     * @param mapper The JSON object mapper.
     * @param map    The representation of SellerGroup as a Map.
     * @return The SellerGroup object.
     */
    public static SellerGroup fromMap(ObjectMapper mapper, final Map<String, Object> map) {
        // map can contain Hint value which is used for UI purpose only we remove this record not to break serialisation
        if (map != null) {
            map.remove("sellerHint");
        }
        return mapper.convertValue(map, SellerGroup.class);
    }

    /**
     * Returns SellerGroup created from a simple fieldName and fieldValue filter.<p>
     * Example:
     * <blockquote>
     * <pre>
     * SellerGroup sellerGroup = SellerGroup.fromSimpleFilter('sellerId', "P12345")
     * </pre>
     * </blockquote>
     *
     * @param sellerFieldName  The name of the field from the Seller Master table used in the simple filter.
     * @param sellerFieldValue The value of the field from the Seller Master table used in the simple filter.
     * @return The SellerGroup object.
     */
    public static SellerGroup fromSimpleFilter(String sellerFieldName, String sellerFieldValue) {
        SellerGroup sg = new SellerGroup();
        sg.setSellerFieldName(sellerFieldName);
        sg.setSellerFieldValue(sellerFieldValue);
        return sg;
    }

    @Override
    public String toString() {
        Filter f = asFilter();
        return "SG[" + (f == null ? super.toString() : f.toString()) + "]";
    }

}
