package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;
import net.pricefx.server.dto.calculation.ContextParameter;

public class CollapseInputBuilder<T> extends AbstractInputBuilder<CollapseInputBuilder, T> {
    public CollapseInputBuilder(InputBuilderContext builderContext, String name) {
        super(builderContext, InputType.COLLAPSE, name);
    }

    @Override
    protected Object _getInput() {
        return null;
    }

    /**
     * Add parameter to the ContextParameters list
     * Example:
     * <blockquote><pre>
     * ContextParameter entry = ...
     *
     * api.inputBuilderFactory()
     *         .createCollapseLayout("Collapsed")
     *         .addInput(entry)
     *         .setCollapsed(true)
     *         .buildContextParameter()
     * </pre></blockquote>
     *
     * @param contextParameter ContextParameter to be added
     * @return self
     */
    public CollapseInputBuilder<T> addInput(ContextParameter contextParameter) {
        cp.addInput(contextParameter);
        return this;
    }

    /**
     * Specifies whether the collapse element in the UI will be shown closed (collapsed). If omitted, the collapse
     * element will be shown opened.
     * Example:
     * <blockquote><pre>
     * ContextParameter entry = ...
     *
     * api.inputBuilderFactory()
     *         .createCollapseLayout("Collapsed")
     *         .addInput(entry)
     *         .setCollapsed(true)
     *         .buildContextParameter()
     * </pre></blockquote>
     *
     * @param isCollapsed true to make the collapse element to be displayed as closed, false to show it opened
     * @return the CollapseInputBuilder instance
     */
    public CollapseInputBuilder<T> setCollapsed(Boolean isCollapsed) {
        cp.setFormattingOption("collapse", isCollapsed);
        return this;
    }
}
