package net.pricefx.formulaengine.scripting.inputbuilder;

import net.pricefx.common.api.InputType;

/**
 * Builder class for input type Configurator or InlineConfigurator
 * ({@link InputType#CONFIGURATOR} or {@link InputType#INLINECONFIGURATOR})
 */
public class ConfiguratorInputBuilder<VT> extends AbstractInputBuilder<ConfiguratorInputBuilder, VT> {

    protected ConfiguratorInputBuilder(InputBuilderContext builderContext, String paramName, String formulaName, boolean inline) {
        super(builderContext, inline ? InputType.INLINECONFIGURATOR : InputType.CONFIGURATOR, paramName);
        cp.setURL(formulaName);
    }

    /**
     * Set particular dimensions for the Configurator (value options table)
     * <p>Example:
     * <blockquote><pre>
     * api.inputBuilderFactory()
     *      .createConfiguratorInputBuilder("configuration", "name", false)
     *      .setDimensions("10", "40")
     *      .buildContextParameter()
     * </pre></blockquote>
     *
     * @param width  width value as a String
     * @param height height value as a String
     * @return self
     */
    public ConfiguratorInputBuilder setDimensions(String width, String height) {
        cp.setValueOptions(new String[]{width, height});
        return this;
    }

}
