package net.pricefx.server.dto.calculation;

import com.fasterxml.jackson.annotation.JsonProperty;
import net.pricefx.formulaengine.scripting.portlets.Portlet;

import java.util.HashMap;
import java.util.Map;

public abstract class AbstractPortletDto implements Portlet {

    @JsonProperty
    protected Map<String, Object> layout;

    public Portlet withWidth(int width, String unit) {
        layout().put("w", value(width, unit));
        return this;
    }

    public Portlet withHeight(int height, String unit) {
        layout().put("h", value(height, unit));
        return this;
    }

    public Portlet withLayout(int width, int height, String unit) {
        layout().put("w", value(width, unit));
        layout().put("h", value(height, unit));
        return this;
    }

    public Portlet withLayout(int width, int height, String wUnit, String hUnit) {
        layout().put("w", value(width, wUnit));
        layout().put("h", value(height, hUnit));
        return this;
    }


    protected Map<String, Object> value(int value, String unit){
        Map<String, Object> valueMap = new HashMap<>();
        valueMap.put("value", value);
        valueMap.put("unit", unit);
        return valueMap;
    }

    protected Map<String, Object> layout() {
        if (layout == null) {
            layout = new HashMap<>();
        }
        return layout;
     }

    @Override
    public Portlet withWidth(int width) {
        return withWidth(width, Portlet.UNIT_PX);
    }

    @Override
    public Portlet withHeight(int height) {
        return withHeight(height, Portlet.UNIT_PX);
    }

    @Override
    public Portlet withLayout(int width, int height) {
        return withLayout(width, height, Portlet.UNIT_PX);
    }

    public void setLayout(Map<String, Object> layout) {
        this.layout = layout;
    }
}
